#include "../Framework/Framework.h"

class MainApp : public OpenGLApp {
protected:
	TextureID spectrum;
	ShaderID mandelbrot;

	double left, right, top, bottom;

public:
	bool processCommand(const char *cmd, String &response);
	void controls();

	bool init();
	bool load();
	bool drawFrame();

	void selectPixelFormat(PixelFormat &pixelFormat);
};

bool MainApp::processCommand(const char *cmd, String &response){
	Tokenizer tok;
	char str[256];

	tok.setString(cmd);
	if (tok.next(str) && stricmp(str, "zoom") == 0){
		unsigned int n = 0;
		double window[4];
		while (tok.next(str) && n < 4){
			if (str[0] != ','){
				if (str[0] != '-'){
					window[n] = atof(str);
				} else {
					tok.next(str);
					window[n] = -atof(str);
				}
				n++;
			}
		}
		if (n < 4){
			response.sprintf("%f, %f, %f, %f", left, top, right, bottom);
		} else {
			left   = window[0];
			top    = window[1];
			right  = window[2];
			bottom = window[3];
		}
	} else {
		return OpenGLApp::processCommand(cmd, response);
	}

	return true;
}


void MainApp::controls(){
	double zoomX = (right - left) * time;
	double zoomY = (bottom - top) * time;

	if (keys[upKey]){
		top    -= zoomY;
		bottom -= zoomY;
	}

	if (keys[downKey]){
		top    += zoomY;
		bottom += zoomY;
	}

	if (keys[leftKey]){
		left  -= zoomX;
		right -= zoomX;
	}

	if (keys[rightKey]){
		left  += zoomX;
		right += zoomX;
	}
	
	if (keys[forwardsKey]){
		left   += zoomX;
		right  -= zoomX;
		top    += zoomY;
		bottom -= zoomY;
	}

	if (keys[backwardsKey]){
		left   -= zoomX;
		right  += zoomX;
		top    -= zoomY;
		bottom += zoomY;
	}

	if (keys[resetKey]){
		left   = -1.85;
		right  =  0.5;
		top    = -1.2;
		bottom =  1.2;
	}
}

void MainApp::selectPixelFormat(PixelFormat &pixelFormat){
	pixelFormat.redBits   = 8;
	pixelFormat.greenBits = 8;
	pixelFormat.blueBits  = 8;
	pixelFormat.alphaBits = 0;
}

bool MainApp::init(){
	left   = -1.85;
	right  =  0.5;
	top    = -1.2;
	bottom =  1.2;

	return true;
}


bool MainApp::load(){
	if (!GL_ARB_fragment_program){
		ALERT("Your driver doesn't support GL_ARB_fragment_program.\nThis demo will now exit.", "Error");
		return false;
	}
	
	spectrum   = renderer->addTexture("../Textures/Spectrum2.tga", TEX_CLAMP);

	mandelbrot = renderer->addFragmentShader(GL_FRAGMENT_PROGRAM_ARB, "Mandelbrot.fsh");

	if (glGetError() != GL_NO_ERROR){
		char *str = (char *) glGetString(GL_PROGRAM_ERROR_STRING_ARB);
		ALERT(str, "Error");
	}

	textTexture  = renderer->addTexture("../Textures/font2.dds");

	return true;
}

bool MainApp::drawFrame(){
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

    glMatrixMode(GL_MODELVIEW);
	glOrtho(0, 1, 1, 0, -1, 1);

	renderer->setTextures(spectrum);
	renderer->setFragmentShader(mandelbrot);
	renderer->apply();

	glBegin(GL_QUADS);
		glTexCoord2d(left,  top);
		glVertex2f(0,0);
		glTexCoord2d(right, top);
		glVertex2f(1,0);
		glTexCoord2d(right, bottom);
		glVertex2f(1,1);
		glTexCoord2d(left,  bottom);
		glVertex2f(0,1);
	glEnd();

	return true;
}

DECLARE_MAIN_APP(MainApp)
